@@ -8,11 +8,7 @@
 	(*pretty-print-spacing* 2)
 	(*pretty-print-float-format* "~,4F")
 	(*pretty-print-left-margin* 0)
-	(any? (lambda (f sequence) ; this and every? ought to be built-in!
-		(call-with-exit
-		 (lambda (return) 
-		   (for-each (lambda (arg) (if (f arg) (return #t))) sequence)
-		   #f)))))
+	(*pretty-print-cycles* #t))
 
     (define pretty-print-1
       (letrec ((messy-number (lambda (z)
@@ -93,6 +89,12 @@
 			(write-char #\) port))
 		      (write (car p) port))))) ; pretty-print? (it's always a symbol)
 	  
+	  (define (any-let-or-hash-table? sequence)
+	    (and (pair? sequence)
+		 (or (let? (car sequence))
+		     (hash-table? (car sequence))
+		     (any-let-or-hash-table? (cdr sequence)))))
+
 	  (let ((writers 
 		 (let ((h (make-hash-table)))
 		   
@@ -434,32 +436,7 @@
 		     (if (rational? obj)
 			 (write obj port)
 			 (display (messy-number obj) port)))
-		    
-		    ((hash-table? obj)
-		     (display "(hash-table" port)
-		     (for-each (lambda (field)
-				 (let ((symstr (object->string (car field))))
-				   (spaces port (+ column 2))
-				   (format port "'(~A . " symstr)
-				   (pretty-print-1 (cdr field) port (+ column 4 (length symstr)))
-				   (write-char #\) port)))
-			       obj)
-		     (write-char #\) port))
-		    
-		    ((let? obj)
-		     (if (and (openlet? obj)
-			      (defined? 'pretty-print obj))
-			 ((obj 'pretty-print) obj port column)
-			 (begin
-			   (display "(inlet" port)
-			   (for-each (lambda (field)
-				       (let ((symstr (symbol->string (car field))))
-					 (spaces port (+ column 5))
-					 (format port ":~A " symstr)
-					 (pretty-print-1 (cdr field) port (+ column 2 (length symstr)))))
-				     obj)
-			   (write-char #\) port))))
-		    
+
 		    ((or (int-vector? obj)
 			 (float-vector? obj))
 		     (if (> (length (vector-dimensions obj)) 1)
@@ -492,6 +469,35 @@
 				     (display numstr port)
 				     (set! col (+ col numlen 1)))))))))
 		    
+		    ((and *pretty-print-cycles*
+			  (pair? (cyclic-sequences obj)))
+		     (format port "~W" obj))
+
+		    ((hash-table? obj)
+		     (display "(hash-table" port)
+		     (for-each (lambda (field)
+				 (let ((symstr (object->string (car field))))
+				   (spaces port (+ column 2))
+				   (format port "'(~A . " symstr)
+				   (pretty-print-1 (cdr field) port (+ column 4 (length symstr)))
+				   (write-char #\) port)))
+			       obj)
+		     (write-char #\) port))
+		    
+		    ((let? obj)
+		     (if (and (openlet? obj)
+			      (defined? 'pretty-print obj))
+			 ((obj 'pretty-print) obj port column)
+			 (begin
+			   (display "(inlet" port)
+			   (for-each (lambda (field)
+				       (let ((symstr (symbol->string (car field))))
+					 (spaces port (+ column 5))
+					 (format port ":~A " symstr)
+					 (pretty-print-1 (cdr field) port (+ column 2 (length symstr)))))
+				     obj)
+			   (write-char #\) port))))
+		    
 		    ((vector? obj)
 		     (if (> (length (vector-dimensions obj)) 1)
 			 (write obj port)
@@ -528,10 +534,7 @@
 		    ((hash-table-ref writers (car obj)) 
 		     => (lambda (f) (f obj port column)))
 		    
-		    ((any? (lambda (p)
-			     (or (hash-table? p)
-				 (let? p)))
-			   obj)
+		    ((any-let-or-hash-table? obj)
 		     (let ((first #t))
 		       (write-char #\( port)
 		       (for-each (lambda (p)
@@ -545,6 +548,15 @@
 			    (strlen (length objstr)))
 		       (if (< (+ column strlen) *pretty-print-length*)
 			   (display objstr port)
+
+			   ;; (format port "~A~A" (cond ((pair-line-number obj) => (lambda (line) (format #f "[~D] " line))) (else "")) objstr)
+			   ;; the other case is above (writers)
+			   ;;    (lambda (f) (format port "[~D]" line) (f obj port column))
+			   ;;    with column check, line check etc
+			   ;; so, the pp/profile hook only needs two calls
+			   ;; (lambda (f obj port column) (f obj port column))
+			   ;; and here (lambda (f obj port column) (f=display obj=objstr port)?
+
 			   (let ((lstlen (length obj)))
 			     
 			     (cond ((or (infinite? lstlen)
@@ -653,9 +665,9 @@
 (define (pp obj)
   (call-with-output-string
    (lambda (p)
-      (if (keyword? obj)
-	  (symbol->string obj)
-	  (pretty-print obj p)))))
+     (if (keyword? obj)
+	 (display obj p)
+	 (pretty-print obj p)))))
 
 #|
 (define (pretty-print-all)
